{
  inputs = {
    nixpkgs.url = "github:NixOS/nixpkgs/nixpkgs-unstable";
    systems.url = "github:nix-systems/default";
    devenv.url = "github:cachix/devenv";
  };

  nixConfig = {
    extra-trusted-public-keys = "devenv.cachix.org-1:w1cLUi8dv3hnoSPGAuibQv+f9TZLr6cv/Hm9XgU50cw=";
    extra-substituters = "https://devenv.cachix.org";
  };

  outputs =
    {
      self,
      systems,
      nixpkgs,
      devenv,
      ...
    }@inputs:
    let
      eachSystem = nixpkgs.lib.genAttrs (import systems);
    in
    {
      packages = eachSystem (system: {
        devenv-up = self.devShells.${system}.default.config.procfileScript;
      });

      devShells = eachSystem (
        system:
        let
          pkgs = nixpkgs.legacyPackages.${system};
        in
        {
          default = devenv.lib.mkShell {
            inherit inputs pkgs;
            modules = [
              (
                { pkgs, ... }:
                {
                  dotenv.disableHint = true;

                  languages.rust.enable = true;

                  packages = with pkgs; [
                    nodejs
                    rustPlatform.bindgenHook

                    nodePackages.pnpm

                    nodePackages.typescript
                    nodePackages.typescript-language-server
                  ];

                  process.managers.process-compose.tui.enable = false;

                  processes = {
                    # ySweet.exec = "pnpm dlx y-sweet@latest serve data";
                  };

                  services.caddy = {
                    enable = true;
                    config = ''
                      {
                        http_port 8000
                        https_port 8443
                      }

                      192.168.10.200 {
                          reverse_proxy localhost:9000
                      }
                    '';
                  };

                  enterShell = ''
                    devenv up -D
                  '';
                }
              )
            ];
          };
        }
      );
    };
}
