import { mdiCog } from "@mdi/js";
import Icon from "@mdi/react";
import { colors } from "~/lib/color";
import { icons } from "~/lib/icon";
import { CollectionSettingsDialog } from "~/components/collection/collection_settings_dialog";
import { Skeleton } from "~/components/ui/skeleton";
import { useCollection } from "~/hooks/use-metadata";
import type { CollectionId } from "~/lib/metadata";

type CollectionHeaderProps = {
    collectionId: CollectionId;
};

export function CollectionHeader(props: CollectionHeaderProps) {
    const collection = useCollection(props.collectionId);

    const color = collection ? colors[collection.get("color")] : undefined;
    const icon = collection ? icons[collection.get("icon")] : undefined;

    return (
        <>
            <div className="w-full bg-sidebar flex flex-row items-center">
                <div className="p-2 flex justify-center items-center">
                    {collection
                        ? (icon && <Icon path={icon.path} size={1.5} color={color!.base} />)
                        : <Skeleton className="w-9 h-9" />
                    }
                </div>
                {collection
                    ? <div className="p-2 flex-grow font-bold text-xl">{collection.get("name")}</div>
                    : <div className="p-2 flex-grow"><Skeleton className="h-7 max-w-full w-[200px]" /></div>
                }
                <CollectionSettingsDialog
                    name={collection?.get("name")}
                    collectionId={props.collectionId}
                >
                    <button
                        className="p-4 flex justify-center items-center hover:bg-sidebar-accent"
                    >
                        <Icon path={mdiCog} size={1} />
                    </button>
                </CollectionSettingsDialog>
            </div>
        </>
    );
}
