import { useState } from "react";
import { type ColorName } from "~/lib/color";
import { type IconName } from "~/lib/icon";
import { ColorPicker } from "../form/color_picker";
import { IconPicker } from "../form/icon_picker";
import { Dialog, DialogClose, DialogContent, DialogDescription, DialogFooter, DialogHeader, DialogTitle, DialogTrigger } from "~/components/ui/dialog";
import { Button } from "~/components/ui/button";
import { Label } from "~/components/ui/label";
import { Input } from "~/components/ui/input";
import { createCollection } from "~/lib/metadata";
import { useCollections } from "~/hooks/use-metadata";
import { useNavigate } from "@tanstack/react-router";

type NewCollectionDialogProps = {
    children: React.ReactNode;
}

export function NewCollectionDialog(props: NewCollectionDialogProps) {
    const collections = useCollections();

    const [name, setName] = useState("");
    const [icon, setIcon] = useState<IconName>();
    const [color, setColor] = useState<ColorName>("white");

    const navigate = useNavigate();
    function submit() {
        const newCollection = createCollection(collections, {
            name,
            icon: icon ?? "",
            color,
        })
        setName("");
        setIcon(undefined);
        setColor("white");

        navigate({ to: "/app/collection/$id", params: { id: newCollection.get("id") } });
    }

    return (
        <Dialog>
            <DialogTrigger asChild>
                {props.children}
            </DialogTrigger>
            <DialogContent>
                <DialogHeader>
                    <DialogTitle>New collection</DialogTitle>
                    <DialogDescription>
                        Settings for the new collection
                    </DialogDescription>
                </DialogHeader>
                <div className="flex flex-col gap-2 p-2">
                    <div className="flex flex-row gap-2">
                        <IconPicker onChange={setIcon} />
                        <div className="grid flex-grow items-center gap-1.5">
                            <Label htmlFor="name">Name</Label>
                            <Input
                                id="name"
                                className="flex-grow"
                                placeholder="Name..."
                                onChange={(name) => setName(name.target.value)}
                            />
                        </div>
                    </div>
                    <ColorPicker onChange={setColor} />
                </div>
                <DialogFooter>
                    <DialogClose asChild>
                        <Button
                            onClick={submit}
                        >Create</Button>
                    </DialogClose>
                </DialogFooter>
            </DialogContent>
        </Dialog>
    );
}
