import { ChevronsUpDown } from "lucide-react";
import { useState } from "react";
import { Button } from "~/components/ui/button";
import { Command, CommandEmpty, CommandGroup, CommandInput, CommandItem, CommandList } from "~/components/ui/command";
import { Popover, PopoverContent, PopoverTrigger } from "~/components/ui/popover";
import { type Color, type ColorName, colors } from "~/lib/color";
import { cn } from "~/lib/utils";

type ColorPickerProps = ({
    onChange?: (color: ColorName) => void;
    withNone?: false;
} | {
    onChange?: (color?: ColorName) => void;
    withNone: true;
}) & {
    initialValue?: ColorName;
    className?: string;
}

export function ColorPicker(props: ColorPickerProps) {
    const [open, setOpen] = useState(false);
    const [value, setValue] = useState<ColorName | undefined>(props.initialValue ?? (props.withNone ? undefined : "white"));

    return (
        <Popover open={open} onOpenChange={setOpen}>
            <PopoverTrigger asChild>
                <Button
                    variant="outline"
                    role="combobox"
                    aria-expanded={open}
                    className={cn("w-[200px] justify-between", props.className)}
                >
                    <Entry color={value ? colors[value] : undefined} />
                    <ChevronsUpDown className="ml-2 h-4 w-4 shrink-0 opacity-50" />
                </Button>
            </PopoverTrigger>
            <PopoverContent className="w-[200px] p-0">
                <Command>
                    <CommandInput placeholder="Search colors..." />
                    <CommandList>
                        <CommandEmpty>No color found.</CommandEmpty>
                        <CommandGroup>
                            {props.withNone && <CommandItem
                                value={"none"}
                                onSelect={() => {
                                    setValue(undefined)
                                    setOpen(false)
                                    props.onChange && props.onChange(undefined)
                                }}
                            >
                                <Entry color={undefined} />
                            </CommandItem>}
                            {Object.entries(colors).map(([key, value]) => (
                                <CommandItem
                                    key={key}
                                    value={key}
                                    onSelect={(currentValue) => {
                                        setValue(currentValue as ColorName)
                                        setOpen(false)
                                        props.onChange && props.onChange(currentValue as ColorName)
                                    }}
                                >
                                    <Entry color={value} />
                                </CommandItem>
                            ))}
                        </CommandGroup>
                    </CommandList>
                </Command>
            </PopoverContent>
        </Popover>
    );
}

function Entry(props: { color?: Color }) {
    return (
        <div className="flex flex-row w-full gap-2 items-center">
            <div
                className="h-4 w-4"
                style={{ backgroundColor: props.color?.base }}
            />
            <span style={{ color: props.color?.hover }}>{props.color?.label ?? "None"}</span>
        </div>
    );
}
