import { useState } from "react"
import { Popover, PopoverContent, PopoverTrigger } from "~/components/ui/popover"
import { ChevronsUpDown } from "lucide-react"
import { properties, type PropertyType } from "~/lib/property"
import { Button } from "~/components/ui/button"
import { Command, CommandEmpty, CommandGroup, CommandInput, CommandItem, CommandList } from "~/components/ui/command"

type PropertyTypeComboboxProps = {
    initialValue: PropertyType
    onSelect: (value: PropertyType) => void
}

export function PropertyTypeCombobox(props: PropertyTypeComboboxProps) {
    const [open, setOpen] = useState(false)
    const [value, setValue] = useState(props.initialValue)

    return (
        <Popover open={open} onOpenChange={setOpen}>
            <PopoverTrigger asChild>
                <Button
                    variant="outline"
                    role="combobox"
                    aria-expanded={open}
                    className="w-[200px] justify-between"
                >
                    {value in properties
                        ? properties[value as PropertyType]
                        : "Select type..."}
                    <ChevronsUpDown className="ml-2 h-4 w-4 shrink-0 opacity-50" />
                </Button>
            </PopoverTrigger>
            <PopoverContent className="w-[200px] p-0">
                <Command>
                    <CommandInput placeholder="Search types..." />
                    <CommandList>
                        <CommandEmpty>No type found.</CommandEmpty>
                        <CommandGroup>
                            {Object.entries(properties).map(([key, value]) => (
                                <CommandItem
                                    key={key}
                                    value={key}
                                    onSelect={(currentValue) => {
                                        setValue(currentValue as PropertyType)
                                        setOpen(false)
                                        props.onSelect(currentValue as PropertyType)
                                    }}
                                >
                                    {value}
                                </CommandItem>
                            ))}
                        </CommandGroup>
                    </CommandList>
                </Command>
            </PopoverContent>
        </Popover>
    )
}
