import type { ExcalidrawImperativeAPI } from "@excalidraw/excalidraw/types";
import { ExcalidrawBinding, yjsToExcalidraw } from "y-excalidraw"
import { Button, Excalidraw } from "@excalidraw/excalidraw";
import * as Y from "yjs";
import { useEffect, useMemo, useRef, useState } from "react";

import "@excalidraw/excalidraw/index.css";
import { useNoteMetadata } from "~/hooks/use-metadata";
import { Settings } from "lucide-react";
import { NoteSettingsDialog } from "./note_settings_dialog";
import { useNoteArray, useNoteAwareness, useNoteMap } from "~/hooks/use-note";
import { useEffectiveTheme } from "~/hooks/use-theme";

export default function NoteCanvas(props: { noteId: string }) {
    const [api, setApi] = useState<ExcalidrawImperativeAPI | null>(null);
    const [binding, setBindings] = useState<ExcalidrawBinding | null>(null);
    const excalidrawRef = useRef(null);
    const metadata = useNoteMetadata(props.noteId);
    const theme = useEffectiveTheme();

    const yElements = useNoteArray("elements") as Y.Array<any>;
    const yAssets = useNoteMap("assets");
    const awareness = useNoteAwareness();

    useEffect(() => {
        if (!api) return;

        const binding = new ExcalidrawBinding(
            yElements,
            yAssets,
            api,
            awareness,
            // excalidraw dom is needed to override the undo/redo buttons in the UI as there is no way to override it via props in excalidraw
            // You might need to pass {trackedOrigins: new Set()} to undomanager depending on whether your provider sets an origin or not
            { excalidrawDom: excalidrawRef.current!, undoManager: new Y.UndoManager(yElements) },
        );
        setBindings(binding);
        return () => {
            setBindings(null);
            binding.destroy();
        };
    }, [api]);

    const initData = {
        elements: yjsToExcalidraw(yElements)
    }

    const excalidraw = useMemo(() => <Excalidraw
        initialData={initData}  // Need to set the initial data
        excalidrawAPI={setApi}
        onPointerUpdate={binding?.onPointerUpdate}
        theme={theme}
        UIOptions={{
            canvasActions: {
                toggleTheme: false,
            },
        }}
        renderTopRightUI={() => (<>
            {metadata && <NoteSettingsDialog noteMetadata={metadata}>
                <Button
                    style={{ width: "2.25rem", height: "2.25rem" }}
                    onSelect={() => { }}
                >
                    <Settings />
                </Button>
            </NoteSettingsDialog>}
        </>)}
    />, []);
    return (
        <div className="w-full h-full" ref={excalidrawRef}>
            {excalidraw}
        </div>
    );
}


