import { Dialog, DialogContent, DialogDescription, DialogHeader, DialogTitle, DialogTrigger } from "~/components/ui/dialog";
import { IconPicker } from "../form/icon_picker";
import { Input } from "~/components/ui/input";
import { Label } from "~/components/ui/label";
import { ColorPicker } from "../form/color_picker";
import type { NoteMetadata } from "~/lib/metadata";

type NoteSettingsDialogProps = {
    noteMetadata: NoteMetadata;
    children: React.ReactNode;
};

export function NoteSettingsDialog(props: NoteSettingsDialogProps) {
    const { noteMetadata } = props;
    if (!noteMetadata) {
        return props.children;
    }

    const primaryColor = noteMetadata.get("primaryColor");
    const secondaryColor = noteMetadata.get("secondaryColor");

    return (
        <Dialog>
            <DialogTrigger asChild>
                {props.children}
            </DialogTrigger>
            <DialogContent>
                <DialogHeader>
                    <DialogTitle>{noteMetadata.get("title")} settings</DialogTitle>
                    <DialogDescription>
                        Settings for {noteMetadata.get("title")}.
                    </DialogDescription>
                </DialogHeader>
                {<div className="flex flex-col gap-2 p-2">
                    <div className="flex flex-row gap-2">
                        <IconPicker withNone
                            onChange={(newIcon) => noteMetadata.set("icon", newIcon ?? "")}
                            initialValue={noteMetadata.get("icon") == "" ? undefined : noteMetadata.get("icon")}
                        />
                        <div className="grid flex-grow items-center gap-1.5">
                            <Label htmlFor="name">Name</Label>
                            <Input
                                id="name"
                                className="flex-grow"
                                placeholder="Name..."
                                defaultValue={noteMetadata.get("title")}
                                onChange={(name) => noteMetadata.set("title", name.target.value)}
                            />
                        </div>
                    </div>
                    <div className="flex flex-row gap-2">
                        <div className="grid flex-grow items-center gap-1.5">
                            <Label>Primary color</Label>
                            <ColorPicker
                                withNone
                                onChange={(color) => noteMetadata.set("primaryColor", color ?? "")}
                                initialValue={primaryColor == "" ? undefined : primaryColor}
                                className="w-full flex-grow"
                            />
                        </div>
                        <div className="grid flex-grow items-center gap-1.5">
                            <Label>Secondary color</Label>
                            <ColorPicker
                                withNone
                                onChange={(color) => noteMetadata.set("secondaryColor", color ?? "")}
                                initialValue={secondaryColor == "" ? undefined : secondaryColor}
                                className="w-full flex-grow"
                            />
                        </div>
                    </div>
                </div>}
            </DialogContent>
        </Dialog>
    );
}
