import { useEffect, useState } from "react";
import { Avatar, AvatarFallback, AvatarImage } from "~/components/ui/avatar";

type User = {
    name?: string;
    email?: string;
    image?: string;
};

type UserAvatarProps = {
    className?: string;
    user?: User;
};

export function UserAvatar(props: UserAvatarProps) {
    const email = props.user?.email ?? undefined;
    const gravatar = useGravatarUrl(email);
    const avatar = props.user?.image ?? gravatar;
    const name = props.user?.name ?? undefined
    const initials = getInitials(name);

    return (
        <Avatar className={props.className}>
            <AvatarImage src={avatar} alt={name} />
            <AvatarFallback className="rounded-lg">{initials}</AvatarFallback>
        </Avatar>
    );
}

function useGravatarUrl(email?: string, size = 80) {
    const [url, setUrl] = useState<string>();
    useEffect(() => {
        if (!email) return;

        const trimmedEmail = email.trim().toLowerCase();
        crypto.subtle.digest('SHA-256', new TextEncoder().encode(trimmedEmail)).then(hashBuffer => {
            const hashArray = Array.from(new Uint8Array(hashBuffer));
            const hashHex = hashArray.map(b => b.toString(16).padStart(2, '0')).join('');
            setUrl(`https://www.gravatar.com/avatar/${hashHex}?s=${size}&d=identicon`);
        });
    }, [email, size]);

    return url;
}

function getInitials(name?: string) {
    if (!name) return "";

    const parts = name.split(" ");
    return parts.map(part => part[0]).join("");
}
