import * as Y from "yjs";
import { useCollections, useNotesMetadata } from "~/hooks/use-metadata";

export function MetadataInspector() {
    const collections = useCollections()
    const notes = useNotesMetadata()

    return <div>
        <span>Collections: <YjsInspector toRender={collections} /></span>
        <span>Notes: <YjsInspector toRender={notes} /></span>
    </div>
}

export function YjsInspector(props: { toRender: any }) {
    return <YjsNode node={props.toRender} depth={0} />
}

type YjsNodeProps<T> = {
    node: T
    depth: number
    label?: string
}

function YjsNode(props: YjsNodeProps<any>) {
    if (props.node instanceof Y.Map) return <YjsMap {...props} />
    if (props.node instanceof Y.Array) return <YjsArray {...props} />
    if (typeof props.node == "string") return <String {...props} />

    return <div>
        {props.label}: {props.node.toString()}
    </div>
}

function YjsMap(props: YjsNodeProps<Y.Map<any>>) {
    const entries = Array.from(props.node.entries())
    return <div>
        {props.label ? `${props.label}: ` : ""}<span>{"{"}<Badge>YMap</Badge></span>
        <div style={{ paddingLeft: "2ch" }}>
            {entries.map(([key, item]) => <YjsNode key={key} node={item} depth={props.depth + 1} label={key} />)}
        </div>
        <span>{"}"}</span>
    </div>
}

function YjsArray(props: YjsNodeProps<Y.Array<any>>) {
    return <div>
        {props.label ? `${props.label}: ` : ""}<span>{"["}<Badge>YArray</Badge></span>
        <div style={{ paddingLeft: "2ch" }}>
            {props.node.map((item, index) => <YjsNode key={index} node={item} depth={props.depth + 1} />)}
        </div>
        <span>{"]"}</span>
    </div>
}

function Badge(props: { children: React.ReactNode }) {
    return <span className="px-1 py-.5 rounded bg-accent ml-1">
        {props.children}
    </span>
}

function String(props: YjsNodeProps<string>) {
    return <div>{props.label}: "{props.node}"</div>
}

