import { ElementNode, type LexicalNode, type SerializedElementNode, type Spread } from "lexical";
import { FocusableNode } from "./focusable_node";
import type { TextFormat } from "../serialized_editor_content";

type SerializedTextFormatNode = Spread<SerializedElementNode, {
    style: TextFormat;
}>;

export class FormattedTextNode extends FocusableNode {
    __internalStyle: TextFormat;

    getStyle(): TextFormat {
        const self = this.getLatest();
        return self.__internalStyle;
    }

    static getType(): string {
        return "formatted-text";
    }

    static clone(node: FormattedTextNode): FormattedTextNode {
        return new FormattedTextNode(node.__internalStyle, node.__key, node.__hasFocus);
    }

    constructor(style: TextFormat, key?: string, hasFocus?: boolean) {
        super(key, hasFocus);
        this.__internalStyle = style;
    }

    isInline(): boolean {
        return true;
    }

    createDOM(): HTMLElement {
        const element = super.createDOM();
        element.classList.add("formatted-text");
        element.classList.add(`format-${this.__internalStyle}`);
        return element;
    }

    updateDOM(old: FormattedTextNode, dom: HTMLElement): boolean {
        return super.updateDOM(old, dom);
    }

    exportJSON(): SerializedTextFormatNode {
        return {
            ...super.exportJSON(),
            type: "formatted-text",
            style: this.__internalStyle,
        };
    }

    static importJSON(serializedNode: SerializedTextFormatNode): FormattedTextNode {
        return $createFormattedTextNode(serializedNode.style);
    }

}

export function $createFormattedTextNode(style: TextFormat) {
    return new FormattedTextNode(style);
}

export function $isFormattedTextNode(node?: LexicalNode | null): node is FormattedTextNode {
    return node instanceof FormattedTextNode;
}

export class FormattedTextMarkerNode extends ElementNode {
    static getType(): string {
        return "formatted-text-marker";
    }

    static clone(node: FormattedTextMarkerNode): FormattedTextMarkerNode {
        return new FormattedTextMarkerNode(node.__key);
    }

    isInline(): boolean {
        return true;
    }

    createDOM(): HTMLElement {
        const element = document.createElement("span");
        element.classList.add("formatted-text-marker");
        return element;
    }

    updateDOM(): boolean {
        return false;
    }

    exportJSON(): SerializedElementNode {
        return {
            ...super.exportJSON(),
            type: "formatted-text-marker",
        };
    }

    static importJSON() {
        return $createFormattedTextMarkerNode();
    }
}

export function $createFormattedTextMarkerNode() {
    return new FormattedTextMarkerNode();
}

export function $isFormattedTextMarkerNode(node?: LexicalNode | null): node is FormattedTextMarkerNode {
    return node instanceof FormattedTextMarkerNode;
}

