import { $createParagraphNode, ElementNode, type LexicalNode, ParagraphNode, type RangeSelection, type SerializedElementNode, type Spread } from "lexical";
import { FocusableNode } from "./focusable_node";

type SerializedHeaderNode = Spread<SerializedElementNode, {
    level: number;
}>;

export class HeaderNode extends FocusableNode {
    __level: number;

    getLevel() {
        const self = this.getLatest();
        return self.__level;
    }

    static getType() {
        return "header";
    }

    static clone(node: HeaderNode) {
        return new HeaderNode(node.__level, node.__key, node.__hasFocus);
    }

    constructor(level: number, key?: string, hasFocus?: boolean) {
        super(key, hasFocus);
        this.__level = level;
    }

    createDOM(): HTMLElement {
        const element = super.createDOM();
        element.classList.add("header", `header-${this.__level}`);
        return element;
    }

    updateDOM(old: HeaderNode, dom: HTMLElement): boolean {
        return super.updateDOM(old, dom);
    }

    insertNewAfter(_selection: RangeSelection, restoreSelection?: boolean): ParagraphNode {
        const newElement = $createParagraphNode();
        this.insertAfter(newElement, restoreSelection);
        return newElement;
    }

    exportJSON(): SerializedHeaderNode {
        return {
            ...super.exportJSON(),
            type: "header",
            level: this.__level,
        };
    }

    static importJSON(serializedHeaderNode: SerializedHeaderNode) {
        return $createHeaderNode(serializedHeaderNode.level);
    }
}

export function $createHeaderNode(level: number) {
    return new HeaderNode(level);
}

export function $isHeaderNode(node?: LexicalNode | null): node is HeaderNode {
    return node instanceof HeaderNode;
}

export class HeaderMarkerNode extends ElementNode {
    static getType() {
        return "header-marker";
    }

    static clone(node: HeaderMarkerNode) {
        return new HeaderMarkerNode(node.__key);
    }

    isInline(): boolean {
        return true;
    }

    createDOM(): HTMLElement {
        const element = document.createElement("span");
        element.classList.add("header-marker");
        return element;
    }

    updateDOM(): boolean {
        return false;
    }

    exportJSON(): SerializedElementNode {
        return {
            ...super.exportJSON(),
            type: "header-marker",
        };
    }

    static importJSON() {
        return $createHeaderMarkerNode();
    }
}

export function $createHeaderMarkerNode() {
    return new HeaderMarkerNode();
}

export function $isHeaderMarkerNode(node?: LexicalNode | null): node is HeaderMarkerNode {
    return node instanceof HeaderMarkerNode;
}

