import { useLexicalComposerContext } from "@lexical/react/LexicalComposerContext";
import { CAN_USE_BEFORE_INPUT, IS_APPLE_WEBKIT, IS_IOS, IS_SAFARI, mergeRegister } from "@lexical/utils";
import { $getSelection, $isRangeSelection, COMMAND_PRIORITY_NORMAL, INSERT_PARAGRAPH_COMMAND, KEY_ENTER_COMMAND } from "lexical";
import { useEffect } from "react";

export function ParagraphPlugin() {
    const [editor] = useLexicalComposerContext();

    useEffect(() => mergeRegister(
        editor.registerCommand(INSERT_PARAGRAPH_COMMAND, () => {
            const selection = $getSelection();
            if (!$isRangeSelection(selection)) {
                return false;
            }
            selection.insertParagraph();
            return true;
        }, COMMAND_PRIORITY_NORMAL),
        editor.registerCommand<KeyboardEvent | null>(
            KEY_ENTER_COMMAND,
            (event) => {
                const selection = $getSelection();
                if (!$isRangeSelection(selection)) {
                    return false;
                }
                if (event !== null) {
                    // If we have beforeinput, then we can avoid blocking
                    // the default behavior. This ensures that the iOS can
                    // intercept that we're actually inserting a paragraph,
                    // and autocomplete, autocapitalize etc work as intended.
                    // This can also cause a strange performance issue in
                    // Safari, where there is a noticeable pause due to
                    // preventing the key down of enter.
                    if (
                        (IS_IOS || IS_SAFARI || IS_APPLE_WEBKIT) &&
                        CAN_USE_BEFORE_INPUT
                    ) {
                        return false;
                    }
                    event.preventDefault();
                }
                return editor.dispatchCommand(INSERT_PARAGRAPH_COMMAND, undefined);
            },
            COMMAND_PRIORITY_NORMAL,
        ),
    ))

    return null;
}
