import React, { useContext } from "react";
import * as Y from "yjs";
import { useYDoc } from "~/hooks/use-ydoc";
import type { CollectionId, CollectionMetadata, CollectionsMetadata, NoteId, NoteMetadata, NotesMetadata } from "~/lib/metadata";
import { useObserve } from "~/hooks/use-observe";

const MetadataContext = React.createContext<Y.Doc | null>(null);

export function MetadataProvider(props: { children: React.ReactNode }) {
    const { ydoc } = useYDoc("metadata");

    return (
        <MetadataContext.Provider value={ydoc} >
            {props.children}
        </MetadataContext.Provider>
    );
}

export function useCollections(): CollectionsMetadata {
    const doc = useContext(MetadataContext)

    if (!doc) {
        throw new Error("useCollections must be used within a MetadataProvider")
    }

    const collections = doc.getArray("collections")
    useObserve(collections, "deep")

    return collections as any as CollectionsMetadata
}

export function useCollection(id: CollectionId | undefined): CollectionMetadata | undefined {
    const collections = useCollections()
    const collection = collections.toArray().find(it => it.get("id") == id)

    return collection
}


export function useNotesMetadata(): NotesMetadata {
    const doc = useContext(MetadataContext)

    if (!doc) {
        throw new Error("useNotes must be used within a MetadataProvider")
    }

    const notes = doc.getArray("notes")
    useObserve(notes, "deep")

    return notes as any as NotesMetadata
}

export function useCollectionNotesMetadata(collectionId: CollectionId | ""): NoteMetadata[] {
    const notes = useNotesMetadata()

    return notes.toArray().filter(it => it.get("collectionId") == collectionId)

}

export function useNoteMetadata(id: NoteId): NoteMetadata | undefined {
    const notes = useNotesMetadata()
    const note = notes.toArray().find(it => it.get("id") == id)

    return note
}
