import { useEffect, useState } from 'react';
import * as Y from 'yjs';
import { IndexeddbPersistence } from 'y-indexeddb'
import { WebsocketProvider } from 'y-websocket';

export function useYDoc(name: string) {
    const [ydoc] = useState<Y.Doc>(() => new Y.Doc());
    const [indexeddbProvider, setIndexeddbProvider] = useState<IndexeddbPersistence | null>(null);
    const [websocketProvider, setWebsocketProvider] = useState<WebsocketProvider | null>(null);

    useEffect(() => {
        if (!ydoc) {
            return
        }

        const [indexeddbProvider, websocketProvider] = createProviders(name, ydoc);
        // Connect the websocket provider. This is not done by default because of lexical.
        websocketProvider.connect();
        setIndexeddbProvider(indexeddbProvider);
        setWebsocketProvider(websocketProvider);

        return () => {
            indexeddbProvider.destroy();
            websocketProvider.destroy();
            ydoc.destroy();
        };
    }, [ydoc]);

    return { ydoc, indexeddbProvider, websocketProvider };
}

export function createProviders(name: string, doc: Y.Doc) {
    const indexeddbProvider = new IndexeddbPersistence(name, doc);
    const websocketProvider = new WebsocketProvider(`/sync`, name, doc, {
        connect: false,
    });


    return [indexeddbProvider, websocketProvider] as const;
}
