import * as Y from "yjs"

import type { ColorName } from "~/lib/color"
import type { IconName } from "~/lib/icon"
import type { YArray, YMap } from "~/lib/yjs"
import type { PropertyType } from "~/lib/property"

export type NoteId = string
export type CollectionId = string
export type PropertyId = string

export type NoteType = "text" | "canvas" | "entity"

export type NotesMetadata = YArray<NoteMetadata>

export type NoteMetadata = YMap<{
    id: NoteId
    collectionId: CollectionId | ""
    title: string
    icon: IconName | ""
    primaryColor: ColorName | ""
    secondaryColor: ColorName | ""
    pinned: boolean
    properties: YArray<NoteProperty>
    type: NoteType
}>

export type NoteProperty = YMap<{
    propertyId: PropertyId
    value: string
}>

export type CollectionsMetadata = YArray<CollectionMetadata>

export type CollectionMetadata = YMap<{
    id: CollectionId
    name: string
    color: ColorName
    icon: IconName | ""
    properties?: YArray<CollectionProperty>
}>

export type CollectionProperty = YMap<{
    id: PropertyId
    name: string
    type: PropertyType
    pinned: boolean
}>

export function createCollection(md: CollectionsMetadata, data: {
    name: string,
    color: ColorName,
    icon: IconName,
}) {
    const collection = new Y.Map() as any as CollectionMetadata;
    collection.set("id", randomUUID());
    collection.set("name", data.name);
    collection.set("color", data.color);
    collection.set("icon", data.icon);
    collection.set("properties", new Y.Array() as any);

    md.push([collection])
    return collection;
}

export function deleteCollection(cmd: CollectionsMetadata, id: CollectionId) {
    const index = cmd.toArray().findIndex(it => it.get("id") == id);
    if (index == -1) {
        return;
    }

    cmd.delete(index, 1)
}

export function createNote(md: NotesMetadata, data: {
    name: string,
    icon: IconName | undefined,
    collectionId: CollectionId | undefined,
    primaryColor: ColorName | undefined,
    secondaryColor: ColorName | undefined,
    type: NoteType
}) {
    const note = new Y.Map() as any as NoteMetadata;
    note.set("id", randomUUID());
    note.set("title", data.name);
    note.set("icon", data.icon ?? "");
    note.set("collectionId", data.collectionId ?? "");
    note.set("primaryColor", data.primaryColor ?? "");
    note.set("secondaryColor", data.secondaryColor ?? "");
    note.set("pinned", false);
    note.set("properties", new Y.Array() as any);
    note.set("type", data.type);

    md.push([note])

    return note;
}

export function randomUUID() {
    return crypto.randomUUID()
}
