import { randomUUID, type CollectionProperty, type NoteProperty, type PropertyId } from "~/lib/metadata";
import type { YArray } from "~/lib/yjs";
import * as Y from "yjs";

export const propertyTypes = ["shortText"] as const;

export type PropertyType = typeof propertyTypes[number];

export const properties: { [P in PropertyType]: string } = {
    "shortText": "Short text",
};

export function createCollectionProperty(md: YArray<CollectionProperty>) {
    const property = new Y.Map() as any as CollectionProperty;
    property.set("id", randomUUID());
    property.set("name", "");
    property.set("type", "shortText");
    property.set("pinned", false);

    md.push([property]);
    return property;
}

export function deleteCollectionProperty(md: YArray<CollectionProperty>, id: PropertyId) {
    const index = md.toArray().findIndex((prop) => prop.get("id") === id);
    if (index === -1) {
        return;
    }

    md.delete(index, 1);
}

export function createNoteProperty(md: YArray<NoteProperty>, data: {
    propertyId: PropertyId;
}) {
    const property = new Y.Map() as any as NoteProperty;
    property.set("propertyId", data.propertyId);
    property.set("value", "");

    md.push([property]);
    return property;
}
