import * as Y from 'yjs'

export type YValue = object | boolean | string | number | Uint8Array | YMap<any> | YArray<any> | Y.Text

type YValueToJSON<T> =
    T extends YMap<any> ? YMapToJSON<T>
    : T extends YArray<any> ? YArrayToJSON<T>
    : T extends Y.Text ? string
    : T

type YMapToJSON<T> = T extends YMap<infer X>
    ? { [KEY in keyof X]: YValueToJSON<X[KEY]> }
    : never;

type YArrayToJSON<T> = T extends YArray<infer X> ? YValueToJSON<X>[] : never;

export type YMap<T extends { [key: string]: YValue }> = {
    set: <X extends keyof T>(key: X, value: T[X]) => void
    get: <X extends keyof T>(key: X) => T[X]
    delete: <X extends keyof T>(key: X) => void
    has: <X extends keyof T>(key: X) => boolean
    clear: () => void
    toJSON: () => YMapToJSON<YMap<T>>
    forEach: (callback: <X extends keyof T>(value: T[X], key: X, map: YMap<T>) => void) => void
    entries: () => Iterator<[string, YValue]>
    values: () => Iterator<YValue>
    keys: () => Iterator<keyof T>
    clone: () => YMap<T>

    size: number
}

export type YArray<T extends YValue> = {
    insert: (index: number, content: T[]) => void
    delete: (index: number, length: number) => void
    push: (values: T[]) => void
    unshift: (content: T[]) => void
    get: (index: number) => T | undefined
    slice: (start: number, end?: number) => T[]
    toArray: () => T[]
    toJSON: () => YArrayToJSON<YArray<T>>
    forEach: (callback: (value: T, index: number, yarray: YArray<T>) => void) => void
    map: <X>(callback: (value: T, index: number, yarray: YArray<T>) => X) => X[]
    clone: () => YArray<T>

    length: number
}

