import { createFileRoute, Outlet } from "@tanstack/react-router";
import { PanelLeft } from "lucide-react";
import { useEffect } from "react";
import { AppSidebar } from "~/components/app_sidebar";
import { Button } from "~/components/ui/button";
import { SidebarInset, SidebarProvider, useSidebar } from "~/components/ui/sidebar";
import { useAuth } from "~/hooks/use-auth";
import { MetadataProvider } from "~/hooks/use-metadata";
import { useIsMobile } from "~/hooks/use-mobile";

export const Route = createFileRoute("/app")({
    component: AppLayout,
});

export function AppLayout() {
    const { isAuthenticated } = useAuth();

    useEffect(() => {
        if (isAuthenticated) {
            return;
        }

        // Redirect to the login page using the browser navigator,
        // since we are leaving the SPA for auth here.
        document.location.href = `/api/auth/login?redirect_url=${encodeURIComponent(document.location.href)}`;
    }, [isAuthenticated]);

    if (!isAuthenticated) {
        return null;
    }

    return (
        <>
            <MetadataProvider>
                <SidebarProvider>
                    <AppSidebar />
                    <div className="flex flex-col h-svh w-full group">
                        <SidebarInset className="flex-grow overflow-scroll">
                            <Outlet />
                        </SidebarInset>
                        <MobileBar />
                    </div>
                </SidebarProvider>
            </MetadataProvider>
        </>
    );
}

function MobileBar() {
    const isMobile = useIsMobile();
    const { toggleSidebar } = useSidebar();

    if (!isMobile) {
        return;
    }

    return (
        <div className="bg-sidebar w-full p-2">
            <Button
                data-sidebar="trigger"
                variant="ghost"
                onClick={toggleSidebar}
                className="justify-start"
            >
                <PanelLeft />
            </Button>
        </div>
    );

}
