import { createFileRoute } from "@tanstack/react-router"
import NoteCanvas from "~/components/note/note_canvas";
import { NoteHeader } from "~/components/note/note_header";
import { Editor } from "~/editor/Editor";
import { useNoteMetadata } from "~/hooks/use-metadata";
import { NoteProvider } from "~/hooks/use-note";
import type { NoteType } from "~/lib/metadata";

export const Route = createFileRoute("/app/note/$id")({
    component: RouteComponent,
})

function RouteComponent() {
    const { id } = Route.useParams();
    const metadata = useNoteMetadata(id);
    const type = metadata?.get("type");

    // Lexical is special and needs to be created before any provider, otherwise
    // it won't load the doc correctly...
    if (type == "text") {
        return <>
            <NoteHeader id={id} />
            <div className="flex-shrink-0 min-h-full max-w-3xl mx-auto w-full">
                <Editor noteId={id} />
            </div>
        </>
    }

    return <NoteProvider id={id}>
        <Content id={id} type={type} />
    </NoteProvider>;
}


function Content(props: { id: string, type?: NoteType }) {
    switch (props.type) {
        case "canvas": return <>
            <NoteCanvas noteId={props.id} />
        </>;
        case "entity": return <>
            Entity type
        </>;
        default: return null;
    }
}
